# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.request_options import RequestOptions
from ....common.types.external_provider import ExternalProvider
from ....common.types.page_token import PageToken
from ....common.types.patient_id import PatientId
from ....common.types.sort_direction import SortDirection
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.mutable_patient import MutablePatient
from .types.mutable_patient_with_mrn import MutablePatientWithMrn
from .types.patient import Patient
from .types.patient_page import PatientPage
from .types.patient_sort_field import PatientSortField

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def create(
        self,
        *,
        request: MutablePatient,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Adds a patient.  VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatient

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.common import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            ExternalProvider,
            HumanName,
            NameUse,
            Relationship,
            Sex,
        )
        from candid.resources.pre_encounter.resources.patients.resources.v_1 import (
            Contact,
            FilingOrder,
            MutablePatient,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.create(
            request=MutablePatient(
                name=HumanName(
                    family="family",
                    given=["given", "given"],
                    use=NameUse.USUAL,
                ),
                other_names=[
                    HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                    HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                ],
                birth_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                biological_sex=Sex.FEMALE,
                primary_address=Address(
                    use=AddressUse.HOME,
                    line=["line", "line"],
                    city="city",
                    state="state",
                    postal_code="postal_code",
                    country="country",
                ),
                other_addresses=[
                    Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                    Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                ],
                primary_telecom=ContactPoint(
                    value="value",
                    use=ContactPointUse.HOME,
                ),
                other_telecoms=[
                    ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                    ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                ],
                contacts=[
                    Contact(
                        relationship=[Relationship.SELF, Relationship.SELF],
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                        ],
                    ),
                    Contact(
                        relationship=[Relationship.SELF, Relationship.SELF],
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                        ],
                    ),
                ],
                general_practitioners=[
                    ExternalProvider(
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                    ),
                    ExternalProvider(
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                    ),
                ],
                filing_order=FilingOrder(
                    coverages=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                    ],
                ),
            ),
        )
        """
        _response = self._raw_client.create(
            request=request, skip_duplicate_check=skip_duplicate_check, request_options=request_options
        )
        return _response.data

    def create_with_mrn(
        self,
        *,
        request: MutablePatientWithMrn,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Adds a patient and hydrates their MRN with a pre-existing MRN.  Once this patient is created their MRN will not be editable. BadRequestError is returned when the MRN is greater than 20 characters. VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatientWithMrn

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.common import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            ExternalProvider,
            HumanName,
            NameUse,
            Relationship,
            Sex,
        )
        from candid.resources.pre_encounter.resources.patients.resources.v_1 import (
            Contact,
            FilingOrder,
            MutablePatientWithMrn,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.create_with_mrn(
            request=MutablePatientWithMrn(
                name=HumanName(
                    family="family",
                    given=["given", "given"],
                    use=NameUse.USUAL,
                ),
                other_names=[
                    HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                    HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                ],
                birth_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                biological_sex=Sex.FEMALE,
                primary_address=Address(
                    use=AddressUse.HOME,
                    line=["line", "line"],
                    city="city",
                    state="state",
                    postal_code="postal_code",
                    country="country",
                ),
                other_addresses=[
                    Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                    Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                ],
                primary_telecom=ContactPoint(
                    value="value",
                    use=ContactPointUse.HOME,
                ),
                other_telecoms=[
                    ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                    ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                ],
                contacts=[
                    Contact(
                        relationship=[Relationship.SELF, Relationship.SELF],
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                        ],
                    ),
                    Contact(
                        relationship=[Relationship.SELF, Relationship.SELF],
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                        ],
                    ),
                ],
                general_practitioners=[
                    ExternalProvider(
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                    ),
                    ExternalProvider(
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                    ),
                ],
                filing_order=FilingOrder(
                    coverages=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                    ],
                ),
                mrn="mrn",
            ),
        )
        """
        _response = self._raw_client.create_with_mrn(
            request=request, skip_duplicate_check=skip_duplicate_check, request_options=request_options
        )
        return _response.data

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        mrn: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        sort_field: typing.Optional[PatientSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientPage:
        """
        Searches for patients that match the query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]

        mrn : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        sort_field : typing.Optional[PatientSortField]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.get_multi()
        """
        _response = self._raw_client.get_multi(
            limit=limit,
            mrn=mrn,
            page_token=page_token,
            sort_field=sort_field,
            sort_direction=sort_direction,
            request_options=request_options,
        )
        return _response.data

    def search_providers(
        self, *, search_criteria: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ExternalProvider]:
        """
        Searches for referring providers that match the query parameters.  The search is case-insensitive, supports fuzzy matching, and matches against provider name and NPI. The search criteria must be an alphanumeric string, and the search is limited to the first 20 results.

        Parameters
        ----------
        search_criteria : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ExternalProvider]

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.search_providers(
            search_criteria="search_criteria",
        )
        """
        _response = self._raw_client.search_providers(search_criteria=search_criteria, request_options=request_options)
        return _response.data

    def get(self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None) -> Patient:
        """
        Gets a patient.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def get_history(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Gets a patient along with it's full history.  The return list is ordered by version ascending.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.get_history(
            id="id",
        )
        """
        _response = self._raw_client.get_history(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: PatientId,
        version: str,
        *,
        request: MutablePatient,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Updates a patient.  The path must contain the most recent version to prevent race conditions.  Updating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request : MutablePatient

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.pre_encounter.resources.common import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            ExternalProvider,
            HumanName,
            NameUse,
            Relationship,
            Sex,
        )
        from candid.resources.pre_encounter.resources.patients.resources.v_1 import (
            Contact,
            FilingOrder,
            MutablePatient,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.update(
            id="id",
            version="version",
            request=MutablePatient(
                name=HumanName(
                    family="family",
                    given=["given", "given"],
                    use=NameUse.USUAL,
                ),
                other_names=[
                    HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                    HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                ],
                birth_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                biological_sex=Sex.FEMALE,
                primary_address=Address(
                    use=AddressUse.HOME,
                    line=["line", "line"],
                    city="city",
                    state="state",
                    postal_code="postal_code",
                    country="country",
                ),
                other_addresses=[
                    Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                    Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                ],
                primary_telecom=ContactPoint(
                    value="value",
                    use=ContactPointUse.HOME,
                ),
                other_telecoms=[
                    ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                    ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                ],
                contacts=[
                    Contact(
                        relationship=[Relationship.SELF, Relationship.SELF],
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                        ],
                    ),
                    Contact(
                        relationship=[Relationship.SELF, Relationship.SELF],
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                        addresses=[
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                            Address(
                                use=AddressUse.HOME,
                                line=["line", "line"],
                                city="city",
                                state="state",
                                postal_code="postal_code",
                                country="country",
                            ),
                        ],
                    ),
                ],
                general_practitioners=[
                    ExternalProvider(
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                    ),
                    ExternalProvider(
                        name=HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        telecoms=[
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                            ContactPoint(
                                value="value",
                                use=ContactPointUse.HOME,
                            ),
                        ],
                    ),
                ],
                filing_order=FilingOrder(
                    coverages=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                    ],
                ),
            ),
        )
        """
        _response = self._raw_client.update(id, version, request=request, request_options=request_options)
        return _response.data

    def deactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Sets a patient as deactivated.  The path must contain the most recent version plus 1 to prevent race conditions.  Deactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.deactivate(
            id="id",
            version="version",
        )
        """
        _response = self._raw_client.deactivate(id, version, request_options=request_options)
        return _response.data

    def reactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Removes the deactivated flag for a patient.  The path must contain the most recent version plus 1 to prevent race conditions.  Reactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.reactivate(
            id="id",
            version="version",
        )
        """
        _response = self._raw_client.reactivate(id, version, request_options=request_options)
        return _response.data

    def search(
        self,
        *,
        mrn: typing.Optional[str] = None,
        similar_name_ordering: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Patient]:
        """
        Returns a list of Patients based on the search criteria.

        Parameters
        ----------
        mrn : typing.Optional[str]

        similar_name_ordering : typing.Optional[str]
            A string that is used to order similar names in search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.search()
        """
        _response = self._raw_client.search(
            mrn=mrn, similar_name_ordering=similar_name_ordering, request_options=request_options
        )
        return _response.data

    def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Scans up to 100 patient updates.  The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import datetime

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.pre_encounter.patients.v_1.scan(
            since=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._raw_client.scan(since=since, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def create(
        self,
        *,
        request: MutablePatient,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Adds a patient.  VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatient

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.common import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            ExternalProvider,
            HumanName,
            NameUse,
            Relationship,
            Sex,
        )
        from candid.resources.pre_encounter.resources.patients.resources.v_1 import (
            Contact,
            FilingOrder,
            MutablePatient,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.create(
                request=MutablePatient(
                    name=HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                    other_names=[
                        HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                    ],
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    biological_sex=Sex.FEMALE,
                    primary_address=Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                    other_addresses=[
                        Address(
                            use=AddressUse.HOME,
                            line=["line", "line"],
                            city="city",
                            state="state",
                            postal_code="postal_code",
                            country="country",
                        ),
                        Address(
                            use=AddressUse.HOME,
                            line=["line", "line"],
                            city="city",
                            state="state",
                            postal_code="postal_code",
                            country="country",
                        ),
                    ],
                    primary_telecom=ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                    other_telecoms=[
                        ContactPoint(
                            value="value",
                            use=ContactPointUse.HOME,
                        ),
                        ContactPoint(
                            value="value",
                            use=ContactPointUse.HOME,
                        ),
                    ],
                    contacts=[
                        Contact(
                            relationship=[Relationship.SELF, Relationship.SELF],
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                            ],
                        ),
                        Contact(
                            relationship=[Relationship.SELF, Relationship.SELF],
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                            ],
                        ),
                    ],
                    general_practitioners=[
                        ExternalProvider(
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                        ),
                        ExternalProvider(
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                        ),
                    ],
                    filing_order=FilingOrder(
                        coverages=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                        ],
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            request=request, skip_duplicate_check=skip_duplicate_check, request_options=request_options
        )
        return _response.data

    async def create_with_mrn(
        self,
        *,
        request: MutablePatientWithMrn,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Adds a patient and hydrates their MRN with a pre-existing MRN.  Once this patient is created their MRN will not be editable. BadRequestError is returned when the MRN is greater than 20 characters. VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatientWithMrn

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.common import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            ExternalProvider,
            HumanName,
            NameUse,
            Relationship,
            Sex,
        )
        from candid.resources.pre_encounter.resources.patients.resources.v_1 import (
            Contact,
            FilingOrder,
            MutablePatientWithMrn,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.create_with_mrn(
                request=MutablePatientWithMrn(
                    name=HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                    other_names=[
                        HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                    ],
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    biological_sex=Sex.FEMALE,
                    primary_address=Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                    other_addresses=[
                        Address(
                            use=AddressUse.HOME,
                            line=["line", "line"],
                            city="city",
                            state="state",
                            postal_code="postal_code",
                            country="country",
                        ),
                        Address(
                            use=AddressUse.HOME,
                            line=["line", "line"],
                            city="city",
                            state="state",
                            postal_code="postal_code",
                            country="country",
                        ),
                    ],
                    primary_telecom=ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                    other_telecoms=[
                        ContactPoint(
                            value="value",
                            use=ContactPointUse.HOME,
                        ),
                        ContactPoint(
                            value="value",
                            use=ContactPointUse.HOME,
                        ),
                    ],
                    contacts=[
                        Contact(
                            relationship=[Relationship.SELF, Relationship.SELF],
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                            ],
                        ),
                        Contact(
                            relationship=[Relationship.SELF, Relationship.SELF],
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                            ],
                        ),
                    ],
                    general_practitioners=[
                        ExternalProvider(
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                        ),
                        ExternalProvider(
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                        ),
                    ],
                    filing_order=FilingOrder(
                        coverages=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                        ],
                    ),
                    mrn="mrn",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_with_mrn(
            request=request, skip_duplicate_check=skip_duplicate_check, request_options=request_options
        )
        return _response.data

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        mrn: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        sort_field: typing.Optional[PatientSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientPage:
        """
        Searches for patients that match the query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]

        mrn : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        sort_field : typing.Optional[PatientSortField]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            limit=limit,
            mrn=mrn,
            page_token=page_token,
            sort_field=sort_field,
            sort_direction=sort_direction,
            request_options=request_options,
        )
        return _response.data

    async def search_providers(
        self, *, search_criteria: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ExternalProvider]:
        """
        Searches for referring providers that match the query parameters.  The search is case-insensitive, supports fuzzy matching, and matches against provider name and NPI. The search criteria must be an alphanumeric string, and the search is limited to the first 20 results.

        Parameters
        ----------
        search_criteria : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ExternalProvider]

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.search_providers(
                search_criteria="search_criteria",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search_providers(
            search_criteria=search_criteria, request_options=request_options
        )
        return _response.data

    async def get(self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None) -> Patient:
        """
        Gets a patient.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def get_history(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Gets a patient along with it's full history.  The return list is ordered by version ascending.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.get_history(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_history(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: PatientId,
        version: str,
        *,
        request: MutablePatient,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Patient:
        """
        Updates a patient.  The path must contain the most recent version to prevent race conditions.  Updating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request : MutablePatient

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Patient

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.pre_encounter.resources.common import (
            Address,
            AddressUse,
            ContactPoint,
            ContactPointUse,
            ExternalProvider,
            HumanName,
            NameUse,
            Relationship,
            Sex,
        )
        from candid.resources.pre_encounter.resources.patients.resources.v_1 import (
            Contact,
            FilingOrder,
            MutablePatient,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.update(
                id="id",
                version="version",
                request=MutablePatient(
                    name=HumanName(
                        family="family",
                        given=["given", "given"],
                        use=NameUse.USUAL,
                    ),
                    other_names=[
                        HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                        HumanName(
                            family="family",
                            given=["given", "given"],
                            use=NameUse.USUAL,
                        ),
                    ],
                    birth_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    biological_sex=Sex.FEMALE,
                    primary_address=Address(
                        use=AddressUse.HOME,
                        line=["line", "line"],
                        city="city",
                        state="state",
                        postal_code="postal_code",
                        country="country",
                    ),
                    other_addresses=[
                        Address(
                            use=AddressUse.HOME,
                            line=["line", "line"],
                            city="city",
                            state="state",
                            postal_code="postal_code",
                            country="country",
                        ),
                        Address(
                            use=AddressUse.HOME,
                            line=["line", "line"],
                            city="city",
                            state="state",
                            postal_code="postal_code",
                            country="country",
                        ),
                    ],
                    primary_telecom=ContactPoint(
                        value="value",
                        use=ContactPointUse.HOME,
                    ),
                    other_telecoms=[
                        ContactPoint(
                            value="value",
                            use=ContactPointUse.HOME,
                        ),
                        ContactPoint(
                            value="value",
                            use=ContactPointUse.HOME,
                        ),
                    ],
                    contacts=[
                        Contact(
                            relationship=[Relationship.SELF, Relationship.SELF],
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                            ],
                        ),
                        Contact(
                            relationship=[Relationship.SELF, Relationship.SELF],
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                            addresses=[
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                                Address(
                                    use=AddressUse.HOME,
                                    line=["line", "line"],
                                    city="city",
                                    state="state",
                                    postal_code="postal_code",
                                    country="country",
                                ),
                            ],
                        ),
                    ],
                    general_practitioners=[
                        ExternalProvider(
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                        ),
                        ExternalProvider(
                            name=HumanName(
                                family="family",
                                given=["given", "given"],
                                use=NameUse.USUAL,
                            ),
                            telecoms=[
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                                ContactPoint(
                                    value="value",
                                    use=ContactPointUse.HOME,
                                ),
                            ],
                        ),
                    ],
                    filing_order=FilingOrder(
                        coverages=[
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                            uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            ),
                        ],
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, version, request=request, request_options=request_options)
        return _response.data

    async def deactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Sets a patient as deactivated.  The path must contain the most recent version plus 1 to prevent race conditions.  Deactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.deactivate(
                id="id",
                version="version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deactivate(id, version, request_options=request_options)
        return _response.data

    async def reactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Removes the deactivated flag for a patient.  The path must contain the most recent version plus 1 to prevent race conditions.  Reactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.reactivate(
                id="id",
                version="version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.reactivate(id, version, request_options=request_options)
        return _response.data

    async def search(
        self,
        *,
        mrn: typing.Optional[str] = None,
        similar_name_ordering: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Patient]:
        """
        Returns a list of Patients based on the search criteria.

        Parameters
        ----------
        mrn : typing.Optional[str]

        similar_name_ordering : typing.Optional[str]
            A string that is used to order similar names in search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            mrn=mrn, similar_name_ordering=similar_name_ordering, request_options=request_options
        )
        return _response.data

    async def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Patient]:
        """
        Scans up to 100 patient updates.  The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Patient]

        Examples
        --------
        import asyncio
        import datetime

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.pre_encounter.patients.v_1.scan(
                since=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.scan(since=since, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.insurance_adjudication import InsuranceAdjudication
from .types.insurance_adjudication_create import InsuranceAdjudicationCreate
from .types.insurance_adjudication_id import InsuranceAdjudicationId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def get(
        self,
        insurance_adjudication_id: InsuranceAdjudicationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InsuranceAdjudication:
        """
        Retrieves a previously created insurance adjudication by its `insurance_adjudication_id`.

        Parameters
        ----------
        insurance_adjudication_id : InsuranceAdjudicationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InsuranceAdjudication

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.insurance_adjudications.v_1.get(
            insurance_adjudication_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(insurance_adjudication_id, request_options=request_options)
        return _response.data

    def create(
        self, *, request: InsuranceAdjudicationCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> InsuranceAdjudication:
        """
        Creates a new insurance adjudication record and returns the newly created InsuranceAdjudication object.

        Parameters
        ----------
        request : InsuranceAdjudicationCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InsuranceAdjudication

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import ClaimAdjustmentGroupCodes
        from candid.resources.era_commons import ClaimStatusCodeCreate
        from candid.resources.insurance_adjudications.resources.v_1 import (
            ClaimAdjudicationCreate,
            InsuranceAdjudicationCreate,
            ServiceLineAdjudicationCreate,
        )
        from candid.resources.payers.resources.v_3 import PayerIdentifier_PayerInfo
        from candid.resources.remits.resources.v_1 import Payee, PayeeIdentifier_Npi
        from candid.resources.x_12.resources.v_1 import (
            Carc,
            ClaimAdjustmentReasonCode,
            Rarc,
            RemittanceAdviceRemarkCode,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.insurance_adjudications.v_1.create(
            request=InsuranceAdjudicationCreate(
                payer_identifier=PayerIdentifier_PayerInfo(
                    payer_id="payer_id",
                    payer_name="payer_name",
                ),
                payee=Payee(
                    payee_name="payee_name",
                    payee_identifier=PayeeIdentifier_Npi(value="payee_identifier"),
                ),
                check_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                claims={
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ): [
                        ClaimAdjudicationCreate(
                            claim_status_code=ClaimStatusCodeCreate.PROCESSED_AS_PRIMARY,
                            service_lines={
                                uuid.UUID(
                                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                                ): [
                                    ServiceLineAdjudicationCreate(
                                        carcs=[
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                        ],
                                        rarcs=[
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                        ],
                                    ),
                                    ServiceLineAdjudicationCreate(
                                        carcs=[
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                        ],
                                        rarcs=[
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                        ],
                                    ),
                                ]
                            },
                            carcs=[
                                ClaimAdjustmentReasonCode(
                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                    reason_code=Carc.CARC_1,
                                    amount_cents=1,
                                ),
                                ClaimAdjustmentReasonCode(
                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                    reason_code=Carc.CARC_1,
                                    amount_cents=1,
                                ),
                            ],
                        ),
                        ClaimAdjudicationCreate(
                            claim_status_code=ClaimStatusCodeCreate.PROCESSED_AS_PRIMARY,
                            service_lines={
                                uuid.UUID(
                                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                                ): [
                                    ServiceLineAdjudicationCreate(
                                        carcs=[
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                        ],
                                        rarcs=[
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                        ],
                                    ),
                                    ServiceLineAdjudicationCreate(
                                        carcs=[
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                            ClaimAdjustmentReasonCode(
                                                group_code=ClaimAdjustmentGroupCodes.CO,
                                                reason_code=Carc.CARC_1,
                                                amount_cents=1,
                                            ),
                                        ],
                                        rarcs=[
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                            RemittanceAdviceRemarkCode(
                                                reason_code=Rarc.M_1,
                                            ),
                                        ],
                                    ),
                                ]
                            },
                            carcs=[
                                ClaimAdjustmentReasonCode(
                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                    reason_code=Carc.CARC_1,
                                    amount_cents=1,
                                ),
                                ClaimAdjustmentReasonCode(
                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                    reason_code=Carc.CARC_1,
                                    amount_cents=1,
                                ),
                            ],
                        ),
                    ]
                },
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def delete(
        self,
        insurance_adjudication_id: InsuranceAdjudicationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes the insurance adjudication record matching the provided insurance_adjudication_id.

        Parameters
        ----------
        insurance_adjudication_id : InsuranceAdjudicationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.insurance_adjudications.v_1.delete(
            insurance_adjudication_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.delete(insurance_adjudication_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def get(
        self,
        insurance_adjudication_id: InsuranceAdjudicationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InsuranceAdjudication:
        """
        Retrieves a previously created insurance adjudication by its `insurance_adjudication_id`.

        Parameters
        ----------
        insurance_adjudication_id : InsuranceAdjudicationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InsuranceAdjudication

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.insurance_adjudications.v_1.get(
                insurance_adjudication_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(insurance_adjudication_id, request_options=request_options)
        return _response.data

    async def create(
        self, *, request: InsuranceAdjudicationCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> InsuranceAdjudication:
        """
        Creates a new insurance adjudication record and returns the newly created InsuranceAdjudication object.

        Parameters
        ----------
        request : InsuranceAdjudicationCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InsuranceAdjudication

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import ClaimAdjustmentGroupCodes
        from candid.resources.era_commons import ClaimStatusCodeCreate
        from candid.resources.insurance_adjudications.resources.v_1 import (
            ClaimAdjudicationCreate,
            InsuranceAdjudicationCreate,
            ServiceLineAdjudicationCreate,
        )
        from candid.resources.payers.resources.v_3 import PayerIdentifier_PayerInfo
        from candid.resources.remits.resources.v_1 import Payee, PayeeIdentifier_Npi
        from candid.resources.x_12.resources.v_1 import (
            Carc,
            ClaimAdjustmentReasonCode,
            Rarc,
            RemittanceAdviceRemarkCode,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.insurance_adjudications.v_1.create(
                request=InsuranceAdjudicationCreate(
                    payer_identifier=PayerIdentifier_PayerInfo(
                        payer_id="payer_id",
                        payer_name="payer_name",
                    ),
                    payee=Payee(
                        payee_name="payee_name",
                        payee_identifier=PayeeIdentifier_Npi(value="payee_identifier"),
                    ),
                    check_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    claims={
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ): [
                            ClaimAdjudicationCreate(
                                claim_status_code=ClaimStatusCodeCreate.PROCESSED_AS_PRIMARY,
                                service_lines={
                                    uuid.UUID(
                                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                                    ): [
                                        ServiceLineAdjudicationCreate(
                                            carcs=[
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                            ],
                                            rarcs=[
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                            ],
                                        ),
                                        ServiceLineAdjudicationCreate(
                                            carcs=[
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                            ],
                                            rarcs=[
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                            ],
                                        ),
                                    ]
                                },
                                carcs=[
                                    ClaimAdjustmentReasonCode(
                                        group_code=ClaimAdjustmentGroupCodes.CO,
                                        reason_code=Carc.CARC_1,
                                        amount_cents=1,
                                    ),
                                    ClaimAdjustmentReasonCode(
                                        group_code=ClaimAdjustmentGroupCodes.CO,
                                        reason_code=Carc.CARC_1,
                                        amount_cents=1,
                                    ),
                                ],
                            ),
                            ClaimAdjudicationCreate(
                                claim_status_code=ClaimStatusCodeCreate.PROCESSED_AS_PRIMARY,
                                service_lines={
                                    uuid.UUID(
                                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                                    ): [
                                        ServiceLineAdjudicationCreate(
                                            carcs=[
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                            ],
                                            rarcs=[
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                            ],
                                        ),
                                        ServiceLineAdjudicationCreate(
                                            carcs=[
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                                ClaimAdjustmentReasonCode(
                                                    group_code=ClaimAdjustmentGroupCodes.CO,
                                                    reason_code=Carc.CARC_1,
                                                    amount_cents=1,
                                                ),
                                            ],
                                            rarcs=[
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                                RemittanceAdviceRemarkCode(
                                                    reason_code=Rarc.M_1,
                                                ),
                                            ],
                                        ),
                                    ]
                                },
                                carcs=[
                                    ClaimAdjustmentReasonCode(
                                        group_code=ClaimAdjustmentGroupCodes.CO,
                                        reason_code=Carc.CARC_1,
                                        amount_cents=1,
                                    ),
                                    ClaimAdjustmentReasonCode(
                                        group_code=ClaimAdjustmentGroupCodes.CO,
                                        reason_code=Carc.CARC_1,
                                        amount_cents=1,
                                    ),
                                ],
                            ),
                        ]
                    },
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def delete(
        self,
        insurance_adjudication_id: InsuranceAdjudicationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes the insurance adjudication record matching the provided insurance_adjudication_id.

        Parameters
        ----------
        insurance_adjudication_id : InsuranceAdjudicationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.insurance_adjudications.v_1.delete(
                insurance_adjudication_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(insurance_adjudication_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .tag import Tag
from .tag_color_enum import TagColorEnum
from .tag_create import TagCreate
from .tag_id import TagId

__all__ = ["Tag", "TagColorEnum", "TagCreate", "TagId"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Relationship(str, enum.Enum):
    SELF = "SELF"
    SPOUSE = "SPOUSE"
    CHILD = "CHILD"
    COMMON_LAW_SPOUSE = "COMMON_LAW_SPOUSE"
    OTHER = "OTHER"
    _UNKNOWN = "__RELATIONSHIP_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Relationship":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        self_: typing.Callable[[], T_Result],
        spouse: typing.Callable[[], T_Result],
        child: typing.Callable[[], T_Result],
        common_law_spouse: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Relationship.SELF:
            return self_()
        if self is Relationship.SPOUSE:
            return spouse()
        if self is Relationship.CHILD:
            return child()
        if self is Relationship.COMMON_LAW_SPOUSE:
            return common_law_spouse()
        if self is Relationship.OTHER:
            return other()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .intake_question import IntakeQuestion
from .patient_history_category_enum import PatientHistoryCategoryEnum


class PatientHistoryCategory(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.resources.v_4 import (
        IntakeFollowUp,
        IntakeQuestion,
        IntakeResponseAndFollowUps,
        PatientHistoryCategory,
        PatientHistoryCategoryEnum,
    )

    PatientHistoryCategory(
        category=PatientHistoryCategoryEnum.PRESENT_ILLNESS,
        questions=[
            IntakeQuestion(
                id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176",
                text="Do you have any allergies?",
                responses=[
                    IntakeResponseAndFollowUps(
                        response="No allergies",
                        follow_ups=[
                            IntakeFollowUp(
                                id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
                                text="Do you have any allergies?",
                                response="No allergies",
                            )
                        ],
                    )
                ],
            )
        ],
    )
    """

    category: PatientHistoryCategoryEnum
    questions: typing.List[IntakeQuestion] = pydantic.Field()
    """
    Must contain at least one item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.encounter_id import EncounterId
from .....commons.types.user_id import UserId
from .....commons.types.work_queue_id import WorkQueueId
from ...commons.types.task_category import TaskCategory
from ...commons.types.task_type import TaskType


class TaskCreateV3(UniversalBaseModel):
    encounter_id: EncounterId
    task_type: TaskType
    description: str
    blocks_claim_submission: typing.Optional[bool] = None
    assignee_user_id: typing.Optional[UserId] = None
    category: typing.Optional[TaskCategory] = None
    work_queue_id: WorkQueueId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CredentialingSpanStatus(str, enum.Enum):
    """
    Stage of a credentialing span in the credentialing workflow.
    """

    WORK_IN_PROGRESS = "work_in_progress"
    PENDING = "pending"
    EFFECTIVE = "effective"
    EXPIRED = "expired"
    _UNKNOWN = "__CREDENTIALINGSPANSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CredentialingSpanStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        work_in_progress: typing.Callable[[], T_Result],
        pending: typing.Callable[[], T_Result],
        effective: typing.Callable[[], T_Result],
        expired: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CredentialingSpanStatus.WORK_IN_PROGRESS:
            return work_in_progress()
        if self is CredentialingSpanStatus.PENDING:
            return pending()
        if self is CredentialingSpanStatus.EFFECTIVE:
            return effective()
        if self is CredentialingSpanStatus.EXPIRED:
            return expired()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.patient_id import PatientId
from .....common.types.relationship import Relationship
from .coverage_benefits import CoverageBenefits
from .coverage_status import CoverageStatus
from .eligibility_check_metadata import EligibilityCheckMetadata
from .insurance_plan import InsurancePlan
from .latest_eligibility_check import LatestEligibilityCheck
from .subscriber import Subscriber


class MutableCoverage(UniversalBaseModel):
    status: CoverageStatus = pydantic.Field()
    """
    The status indiciating if the coverage is active or not.
    """

    subscriber: Subscriber = pydantic.Field()
    """
    The party who has signed-up for or 'owns' the contractual relationship to the policy or to whom the benefit of the policy for services rendered to them or their family is due.
    """

    relationship: Relationship = pydantic.Field()
    """
    The relationship of beneficiary (patient) to the subscriber. https://hl7.org/fhir/valueset-relationship.html
    """

    patient: PatientId = pydantic.Field()
    """
    The patient who benefits from the insurance coverage
    """

    insurance_plan: InsurancePlan
    verified: bool = pydantic.Field()
    """
    A boolean indicating if the coverage has been verified by a user.
    """

    eligibility_checks: typing.Optional[typing.List[EligibilityCheckMetadata]] = pydantic.Field(default=None)
    """
    A list of eligibility check metadata that have been initiated on this coverage.
    """

    latest_eligibility_check: typing.Optional[LatestEligibilityCheck] = pydantic.Field(default=None)
    """
    The latest eligibility check metadata that has been initiated on this coverage.
    """

    benefits: typing.Optional[CoverageBenefits] = pydantic.Field(default=None)
    """
    The eligibility of the patient for the coverage, manually verified by users.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .cash_pay_payer_error import CashPayPayerError
from .encounter_external_id_uniqueness_error import EncounterExternalIdUniquenessError
from .encounter_guarantor_missing_contact_info_error import EncounterGuarantorMissingContactInfoError
from .encounter_patient_control_number_uniqueness_error import EncounterPatientControlNumberUniquenessError
from .invalid_tag_names_error import InvalidTagNamesError
from .payer_plan_group_payer_does_not_match_insurance_card_http_error import (
    PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError,
)
from .schema_instance_validation_http_failure import SchemaInstanceValidationHttpFailure

__all__ = [
    "CashPayPayerError",
    "EncounterExternalIdUniquenessError",
    "EncounterGuarantorMissingContactInfoError",
    "EncounterPatientControlNumberUniquenessError",
    "InvalidTagNamesError",
    "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError",
    "SchemaInstanceValidationHttpFailure",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from .....payers.resources.v_3.types.payer_uuid import PayerUuid
from .explanation import Explanation


class ComputeAllInNetworkRenderingProvidersResult_RenderingProviders(UniversalBaseModel):
    type: typing.Literal["rendering_providers"] = "rendering_providers"
    rendering_providers: typing.List[OrganizationProviderId]
    routed_payer_uuid: PayerUuid
    routed_billing_provider_id: OrganizationProviderId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ComputeAllInNetworkRenderingProvidersResult_Indeterminate(UniversalBaseModel):
    type: typing.Literal["indeterminate"] = "indeterminate"
    error: str
    explanation: Explanation
    routed_payer_uuid: typing.Optional[PayerUuid] = None
    routed_billing_provider_id: typing.Optional[OrganizationProviderId] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ComputeAllInNetworkRenderingProvidersResult_OutOfNetwork(UniversalBaseModel):
    type: typing.Literal["out_of_network"] = "out_of_network"
    explanation: Explanation
    routed_payer_uuid: PayerUuid
    routed_billing_provider_id: OrganizationProviderId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ComputeAllInNetworkRenderingProvidersResult = typing.Union[
    ComputeAllInNetworkRenderingProvidersResult_RenderingProviders,
    ComputeAllInNetworkRenderingProvidersResult_Indeterminate,
    ComputeAllInNetworkRenderingProvidersResult_OutOfNetwork,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.claim_id import ClaimId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.provider_id import ProviderId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....payers.resources.v_3.types.payer_uuid import PayerUuid
from .types.insurance_payment import InsurancePayment
from .types.insurance_payment_id import InsurancePaymentId
from .types.insurance_payment_sort_field import InsurancePaymentSortField
from .types.insurance_payments_page import InsurancePaymentsPage


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        sort: typing.Optional[InsurancePaymentSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[InsurancePaymentsPage]:
        """
        Returns all non-ERA originated insurance payments satisfying the search criteria

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        payer_uuid : typing.Optional[PayerUuid]

        claim_id : typing.Optional[ClaimId]

        service_line_id : typing.Optional[ServiceLineId]

        billing_provider_id : typing.Optional[ProviderId]

        sort : typing.Optional[InsurancePaymentSortField]
            Defaults to payment_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InsurancePaymentsPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/insurance-payments/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "payer_uuid": payer_uuid,
                "claim_id": claim_id,
                "service_line_id": service_line_id,
                "billing_provider_id": billing_provider_id,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                InsurancePaymentsPage,
                parse_obj_as(
                    type_=InsurancePaymentsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, insurance_payment_id: InsurancePaymentId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InsurancePayment]:
        """
        Retrieves a previously created insurance payment by its `insurance_payment_id`.
        If the payment does not exist, a `403` will be thrown.

        Parameters
        ----------
        insurance_payment_id : InsurancePaymentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InsurancePayment]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/insurance-payments/v1/{jsonable_encoder(insurance_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                InsurancePayment,
                parse_obj_as(
                    type_=InsurancePayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        payer_uuid: typing.Optional[PayerUuid] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        sort: typing.Optional[InsurancePaymentSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[InsurancePaymentsPage]:
        """
        Returns all non-ERA originated insurance payments satisfying the search criteria

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        payer_uuid : typing.Optional[PayerUuid]

        claim_id : typing.Optional[ClaimId]

        service_line_id : typing.Optional[ServiceLineId]

        billing_provider_id : typing.Optional[ProviderId]

        sort : typing.Optional[InsurancePaymentSortField]
            Defaults to payment_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InsurancePaymentsPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/insurance-payments/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "payer_uuid": payer_uuid,
                "claim_id": claim_id,
                "service_line_id": service_line_id,
                "billing_provider_id": billing_provider_id,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                InsurancePaymentsPage,
                parse_obj_as(
                    type_=InsurancePaymentsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, insurance_payment_id: InsurancePaymentId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InsurancePayment]:
        """
        Retrieves a previously created insurance payment by its `insurance_payment_id`.
        If the payment does not exist, a `403` will be thrown.

        Parameters
        ----------
        insurance_payment_id : InsurancePaymentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InsurancePayment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/insurance-payments/v1/{jsonable_encoder(insurance_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                InsurancePayment,
                parse_obj_as(
                    type_=InsurancePayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AppointmentWorkQueue(str, enum.Enum):
    EMERGENT_ISSUE = "EMERGENT_ISSUE"
    NEW_PATIENT = "NEW_PATIENT"
    RETURNING_PATIENT = "RETURNING_PATIENT"
    MANUAL_ESCALATION = "MANUAL_ESCALATION"
    _UNKNOWN = "__APPOINTMENTWORKQUEUE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AppointmentWorkQueue":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        emergent_issue: typing.Callable[[], T_Result],
        new_patient: typing.Callable[[], T_Result],
        returning_patient: typing.Callable[[], T_Result],
        manual_escalation: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AppointmentWorkQueue.EMERGENT_ISSUE:
            return emergent_issue()
        if self is AppointmentWorkQueue.NEW_PATIENT:
            return new_patient()
        if self is AppointmentWorkQueue.RETURNING_PATIENT:
            return returning_patient()
        if self is AppointmentWorkQueue.MANUAL_ESCALATION:
            return manual_escalation()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .compute_all_in_network_rendering_providers_result import ComputeAllInNetworkRenderingProvidersResult
from .network_status_check_id import NetworkStatusCheckId


class ComputeAllInNetworkProvidersResponse(UniversalBaseModel):
    computed_network_status: ComputeAllInNetworkRenderingProvidersResult
    network_status_check_id: NetworkStatusCheckId = pydantic.Field()
    """
    Unique network status check ID for this request
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .billing_provider import BillingProvider
from .billing_provider_update import BillingProviderUpdate
from .billing_provider_update_with_optional_address import BillingProviderUpdateWithOptionalAddress
from .encounter_provider import EncounterProvider
from .encounter_provider_base import EncounterProviderBase
from .initial_referring_provider import InitialReferringProvider
from .initial_referring_provider_update import InitialReferringProviderUpdate
from .initial_referring_provider_update_with_optional_address import InitialReferringProviderUpdateWithOptionalAddress
from .ordering_provider import OrderingProvider
from .ordering_provider_update import OrderingProviderUpdate
from .provider_id import ProviderId
from .referring_provider import ReferringProvider
from .referring_provider_update import ReferringProviderUpdate
from .referring_provider_update_with_optional_address import ReferringProviderUpdateWithOptionalAddress
from .rendering_provider import RenderingProvider
from .rendering_provider_update import RenderingProviderUpdate
from .rendering_provider_update_with_optional_address import RenderingProviderUpdateWithOptionalAddress
from .supervising_provider import SupervisingProvider
from .supervising_provider_update import SupervisingProviderUpdate
from .supervising_provider_update_with_optional_address import SupervisingProviderUpdateWithOptionalAddress

__all__ = [
    "BillingProvider",
    "BillingProviderUpdate",
    "BillingProviderUpdateWithOptionalAddress",
    "EncounterProvider",
    "EncounterProviderBase",
    "InitialReferringProvider",
    "InitialReferringProviderUpdate",
    "InitialReferringProviderUpdateWithOptionalAddress",
    "OrderingProvider",
    "OrderingProviderUpdate",
    "ProviderId",
    "ReferringProvider",
    "ReferringProviderUpdate",
    "ReferringProviderUpdateWithOptionalAddress",
    "RenderingProvider",
    "RenderingProviderUpdate",
    "RenderingProviderUpdateWithOptionalAddress",
    "SupervisingProvider",
    "SupervisingProviderUpdate",
    "SupervisingProviderUpdateWithOptionalAddress",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import PatientRefund, PatientRefundId, PatientRefundSortField, PatientRefundsPage

__all__ = ["PatientRefund", "PatientRefundId", "PatientRefundSortField", "PatientRefundsPage"]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.types.page_token import PageToken
from .types.payer import Payer
from .types.payer_page import PayerPage
from .types.payer_uuid import PayerUuid


class RawV3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, payer_uuid: PayerUuid, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Payer]:
        """
        Parameters
        ----------
        payer_uuid : PayerUuid

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Payer]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/payers/v3/{jsonable_encoder(payer_uuid)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Payer,
                parse_obj_as(
                    type_=Payer,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayerPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        search_term : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/payers/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "search_term": search_term,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPage,
                parse_obj_as(
                    type_=PayerPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, payer_uuid: PayerUuid, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Payer]:
        """
        Parameters
        ----------
        payer_uuid : PayerUuid

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Payer]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/payers/v3/{jsonable_encoder(payer_uuid)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Payer,
                parse_obj_as(
                    type_=Payer,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayerPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        search_term : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/payers/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "search_term": search_term,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerPage,
                parse_obj_as(
                    type_=PayerPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .match_cpt_code import MatchCptCode
from .match_date import MatchDate
from .match_facility_type_code import MatchFacilityTypeCode
from .match_geo import MatchGeo
from .match_license_type import MatchLicenseType
from .match_modifiers import MatchModifiers
from .match_network_types import MatchNetworkTypes
from .match_payer import MatchPayer
from .match_payer_plan_groups import MatchPayerPlanGroups
from .match_provider import MatchProvider


class DimensionMatch(UniversalBaseModel):
    """
    Dimension matching for a service line
    """

    payer: MatchPayer
    geography: MatchGeo
    organization_billing_provider: MatchProvider
    date_of_service: MatchDate
    cpt_code: MatchCptCode
    modifiers: MatchModifiers
    license_type: MatchLicenseType
    facility_type_code: MatchFacilityTypeCode
    network_types: MatchNetworkTypes
    payer_plan_groups: MatchPayerPlanGroups

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.datetime_utils import serialize_datetime
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.errors.bad_request_error import BadRequestError
from ....common.errors.not_found_error import NotFoundError
from ....common.errors.version_conflict_error import VersionConflictError
from ....common.types.error_base_4_xx import ErrorBase4Xx
from ....common.types.external_provider import ExternalProvider
from ....common.types.page_token import PageToken
from ....common.types.patient_id import PatientId
from ....common.types.sort_direction import SortDirection
from ....common.types.version_conflict_error_body import VersionConflictErrorBody
from .types.mutable_patient import MutablePatient
from .types.mutable_patient_with_mrn import MutablePatientWithMrn
from .types.patient import Patient
from .types.patient_page import PatientPage
from .types.patient_sort_field import PatientSortField

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        request: MutablePatient,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Patient]:
        """
        Adds a patient.  VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatient

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Patient]
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            params={
                "skip_duplicate_check": skip_duplicate_check,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_with_mrn(
        self,
        *,
        request: MutablePatientWithMrn,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Patient]:
        """
        Adds a patient and hydrates their MRN with a pre-existing MRN.  Once this patient is created their MRN will not be editable. BadRequestError is returned when the MRN is greater than 20 characters. VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatientWithMrn

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Patient]
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1/with_mrn",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            params={
                "skip_duplicate_check": skip_duplicate_check,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        mrn: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        sort_field: typing.Optional[PatientSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PatientPage]:
        """
        Searches for patients that match the query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]

        mrn : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        sort_field : typing.Optional[PatientSortField]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatientPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1/get_multi",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "limit": limit,
                "mrn": mrn,
                "page_token": page_token,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientPage,
                parse_obj_as(
                    type_=PatientPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search_providers(
        self, *, search_criteria: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[ExternalProvider]]:
        """
        Searches for referring providers that match the query parameters.  The search is case-insensitive, supports fuzzy matching, and matches against provider name and NPI. The search criteria must be an alphanumeric string, and the search is limited to the first 20 results.

        Parameters
        ----------
        search_criteria : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[ExternalProvider]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1/search_providers",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "search_criteria": search_criteria,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[ExternalProvider],
                parse_obj_as(
                    type_=typing.List[ExternalProvider],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Patient]:
        """
        Gets a patient.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Patient]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_history(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Patient]]:
        """
        Gets a patient along with it's full history.  The return list is ordered by version ascending.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Patient]]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Patient],
                parse_obj_as(
                    type_=typing.List[Patient],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: PatientId,
        version: str,
        *,
        request: MutablePatient,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Patient]:
        """
        Updates a patient.  The path must contain the most recent version to prevent race conditions.  Updating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request : MutablePatient

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Patient]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Sets a patient as deactivated.  The path must contain the most recent version plus 1 to prevent race conditions.  Deactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def reactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Removes the deactivated flag for a patient.  The path must contain the most recent version plus 1 to prevent race conditions.  Reactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PATCH",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search(
        self,
        *,
        mrn: typing.Optional[str] = None,
        similar_name_ordering: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[Patient]]:
        """
        Returns a list of Patients based on the search criteria.

        Parameters
        ----------
        mrn : typing.Optional[str]

        similar_name_ordering : typing.Optional[str]
            A string that is used to order similar names in search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Patient]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "mrn": mrn,
                "similar_name_ordering": similar_name_ordering,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Patient],
                parse_obj_as(
                    type_=typing.List[Patient],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Patient]]:
        """
        Scans up to 100 patient updates.  The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Patient]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "patients/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "since": serialize_datetime(since),
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Patient],
                parse_obj_as(
                    type_=typing.List[Patient],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        request: MutablePatient,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Patient]:
        """
        Adds a patient.  VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatient

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Patient]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            params={
                "skip_duplicate_check": skip_duplicate_check,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_with_mrn(
        self,
        *,
        request: MutablePatientWithMrn,
        skip_duplicate_check: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Patient]:
        """
        Adds a patient and hydrates their MRN with a pre-existing MRN.  Once this patient is created their MRN will not be editable. BadRequestError is returned when the MRN is greater than 20 characters. VersionConflictError is returned when the patient's external ID is already in use.

        Parameters
        ----------
        request : MutablePatientWithMrn

        skip_duplicate_check : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Patient]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1/with_mrn",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            params={
                "skip_duplicate_check": skip_duplicate_check,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        mrn: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        sort_field: typing.Optional[PatientSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PatientPage]:
        """
        Searches for patients that match the query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]

        mrn : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        sort_field : typing.Optional[PatientSortField]

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatientPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1/get_multi",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "limit": limit,
                "mrn": mrn,
                "page_token": page_token,
                "sort_field": sort_field,
                "sort_direction": sort_direction,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PatientPage,
                parse_obj_as(
                    type_=PatientPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search_providers(
        self, *, search_criteria: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[ExternalProvider]]:
        """
        Searches for referring providers that match the query parameters.  The search is case-insensitive, supports fuzzy matching, and matches against provider name and NPI. The search criteria must be an alphanumeric string, and the search is limited to the first 20 results.

        Parameters
        ----------
        search_criteria : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[ExternalProvider]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1/search_providers",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "search_criteria": search_criteria,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[ExternalProvider],
                parse_obj_as(
                    type_=typing.List[ExternalProvider],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Patient]:
        """
        Gets a patient.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Patient]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_history(
        self, id: PatientId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Patient]]:
        """
        Gets a patient along with it's full history.  The return list is ordered by version ascending.

        Parameters
        ----------
        id : PatientId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Patient]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Patient],
                parse_obj_as(
                    type_=typing.List[Patient],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: PatientId,
        version: str,
        *,
        request: MutablePatient,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Patient]:
        """
        Updates a patient.  The path must contain the most recent version to prevent race conditions.  Updating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request : MutablePatient

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Patient]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Patient,
                parse_obj_as(
                    type_=Patient,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Sets a patient as deactivated.  The path must contain the most recent version plus 1 to prevent race conditions.  Deactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def reactivate(
        self, id: PatientId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Removes the deactivated flag for a patient.  The path must contain the most recent version plus 1 to prevent race conditions.  Reactivating historic versions is not supported.

        Parameters
        ----------
        id : PatientId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"patients/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PATCH",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search(
        self,
        *,
        mrn: typing.Optional[str] = None,
        similar_name_ordering: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[Patient]]:
        """
        Returns a list of Patients based on the search criteria.

        Parameters
        ----------
        mrn : typing.Optional[str]

        similar_name_ordering : typing.Optional[str]
            A string that is used to order similar names in search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Patient]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "mrn": mrn,
                "similar_name_ordering": similar_name_ordering,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Patient],
                parse_obj_as(
                    type_=typing.List[Patient],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Patient]]:
        """
        Scans up to 100 patient updates.  The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Patient]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "patients/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "since": serialize_datetime(since),
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Patient],
                parse_obj_as(
                    type_=typing.List[Patient],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ExpectedNetworkStatus(str, enum.Enum):
    IN_NETWORK = "in_network"
    OUT_OF_NETWORK = "out_of_network"
    UNKNOWN = "unknown"
    _UNKNOWN = "__EXPECTEDNETWORKSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ExpectedNetworkStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        in_network: typing.Callable[[], T_Result],
        out_of_network: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ExpectedNetworkStatus.IN_NETWORK:
            return in_network()
        if self is ExpectedNetworkStatus.OUT_OF_NETWORK:
            return out_of_network()
        if self is ExpectedNetworkStatus.UNKNOWN:
            return unknown()
        return _unknown_member(self._value_)
